local tablet = AceLibrary("Tablet-2.0")
local dewdrop = AceLibrary("Dewdrop-2.0")
local L = AceLibrary("AceLocale-2.2"):new("CustomMenuFu")

CustomMenuFu = AceLibrary("AceAddon-2.0"):new("FuBarPlugin-2.0", "AceDB-2.0", "AceConsole-2.0")

CustomMenuFu:RegisterDB("CustomMenuFuDB", "CustomMenuFuCharDB")
CustomMenuFu:RegisterDefaults("profile", {
	entries		= {},
	autosort	= false,
	title		= 'CustomMenuFu',
	hidemenutitle	= true,
})

local menutitle = function() return CustomMenuFu.db.profile and CustomMenuFu.db.profile.title or 'CustomMenuFu' end

CustomMenuFu.title		= menutitle()
CustomMenuFu.name		= 'FuBar - ' .. CustomMenuFu.title
CustomMenuFu.hideMenuTitle	= function() return CustomMenuFu.db.profile.hidemenutitle end
CustomMenuFu.category		= 'Interface Enhancements'
CustomMenuFu.hasIcon		= "Interface\\Icons\\INV_Scroll_08"
CustomMenuFu.hasNoText		= false
CustomMenuFu.hasNoColor		= false
CustomMenuFu.clickableTooltip	= true
-- CustomMenuFu.defaultPosition	= 'CENTER'


tablet:SetFontSizePercent(FuBar, 1.8)


local options = {
	type = 'group',

	args = {
		--[[ actions ]]--
		title = {
			type	= 'text',
			name	= 'Set the menu title',
			desc	= 'Set the title of your CustomMenuFu',
			set	= function(title)
					CustomMenuFu.db.profile.title	= title
					CustomMenuFu.title		= CustomMenuFu.db.profile.title

					CustomMenuFu:UpdateDisplay()
				end,
			get	= function() return CustomMenuFu.title end,
			usage	= '<title> - set the title of your CustomMenuFu',
			},

		add = {
			type 	= "text",
			name 	= "Add / Update",
			desc 	= "Add or update a new menu item.",
			usage 	= "<name> # <script>  --  eg: /menu <add|update> Test # (add a script of \"---\" to create a blank line) DEFAULT_CHAT_FRAME:AddMessage('test')",
			get 	= false,
			set	= function(v) CustomMenuFu:AddItem(v) end,
			validate = function(v) return string.find(v, "(.*)%s+#%s+(.*)") end,
			order	= 110,
			alias	= 'update',
			},

		remove = {
			type 	= "text",
			name 	= "Remove",
			desc 	= "Remove a menu item.",
			usage	= "<name> - remove the menu entry <name>",
			get 	= false,
			set 	= function(entry) CustomMenuFu:RemoveItem(entry) end,
			order	= 120,
			alias	= { 'delete', 'del' },
			},

		show = {
			type	= 'text',
			name	= 'Show',
			desc	= 'Show the code defined for a particular menu entry',
			usage	= '<name> - show the code defined for the menu entry <name>',
			get	= false,
			set	= function(entry) CustomMenuFu:ShowEntry(entry) end,
			order	= 130,
			},

		list = {
			type	= 'execute',
			func	= function() return CustomMenuFu:List() end,
			name	= 'List entries',
			desc	= 'Lists currently defined entries for your CustomMenu in the chat frame',
			order	= 140,
			},

		clearall = {
			type	= 'execute',
			name	= 'Clear all menu entries',
			func	= function()
					CustomMenuFu.db.profile.backup = CustomMenuFu.db.profile.entries
					CustomMenuFu.db.profile.entries = {}
				  end,
			desc	= 'Clears all entries from your custom menu',
			order	= 145,
			confirm	= 'This will clear all the entries in your custom menu.\nAre you sure you wish to do this?',
			},

		sort = {
			type	= 'execute',
			func	= function() return CustomMenuFu:Sort(CustomMenuFu.db.profile.entries) end,
			name	= 'Sort entries',
			desc	= 'Sorts CustomMenuFu entries alphabetically',
			order	= 150,
			},


		blankheaderone = { type = 'header',
			order	= 199,
			},

		--[[ options ]]--
		optionsheader = { type = 'header',
			name	= 'Options',
			order	= 200,
			},

		autosort = {
			type	= 'toggle',
			set	= function() CustomMenuFu.db.profile.autosort = not CustomMenuFu.db.profile.autosort return CustomMenuFu.db.profile.autosort end,
			get	= function() return CustomMenuFu.db.profile.autosort end,
			name	= 'Autosort',
			desc	= 'Toggle automatic sorting of the entries in your CustomMenuFu when adding or updating',
			order	= 210,
			},

		togglemenutitle = {
			type	= 'toggle',
			name	= 'Show menu title',
			desc	= "Toggle the display of your CustomMenuFu's title in the tooltip",
			get	= function() return not CustomMenuFu.hideMenuTitle end,
			set	= function()
					CustomMenuFu.hideMenuTitle		= not CustomMenuFu.hideMenuTitle
					CustomMenuFu.db.profile.hidemenutitle	= CustomMenuFu.hideMenuTitle

					return CustomMenuFu.hideMenuTitle
				end,
			order	= 240,
			},


		--[[
		toggleicon = {
			type	= 'toggle',
			name	= 'Show the icon in FuBar',
			desc	= "Toggle the display of CustomMenuFu's icon in FuBar",
			get	= function() return CustomMenuFu:IsIconShown() end,
			set	= function() return CustomMenuFu:ToggleIconShown() end,
			order	= 220,
			},

		toggletext = {
			type	= 'toggle',
			name	= 'Show text in FuBar',
			desc	= "Toggle the display of CustomMenuFu's title text in FuBar",
			get	= function() return CustomMenuFu:IsTextShown() end,
			set	= function() return CustomMenuFu:ToggleTextShown() end,
			order	= 230,
			},
		]]
		},
	}

CustomMenuFu:RegisterChatCommand({'/cmf', '/custommenufu', '/custommenu', '/menu'}, options)

CustomMenuFu.OnMenuRequest = options

function CustomMenuFu:OnReceiveDrag()
	self:Say('hi')
end

function CustomMenuFu:AddItem(args)
	-- local _, _, name, script = string.find(args, "([^#]+)#(.+)")
	local name, script

	name, script	= strsplit('#', args, 2)

	self:Say('name: ' .. (name or ''))
	self:Say('script: ' .. (script or ''))

	if not (name and script) then return false end

	name	= strtrim(name)
	script	= strtrim(script)

	if (name == '') or (script == '') then return false end

	if self.db.profile.entries[name] then
		self.db.profile.entries[name] = script
		self:Say(string.format('Updated entry: |cff96b4ff%s|r', name))
	else
		self.db.profile.entries[name] = script
		self:Say(string.format('Added entry: |cff96b4ff%s|r', name))
	end

	if self.db.profile.autosort then
		self.db.profile.entries = self:Sort(self.db.profile.entries)
	end

	self:UpdateDisplay()
end

function CustomMenuFu:Sort(entries)
	entries	= entries or {}

	table.sort(entries)

	self:UpdateDisplay()

	return entries
end

function CustomMenuFu:RemoveItem(entry)
	entry	= entry or ""
	entry	= strtrim(entry)

	if not self.db.profile.entries[entry] then
		self:Say(string.format('No such menu entry: |cff96b4ff%s|r', entry))

		return false
	else
		self.db.profile.entries[entry] = nil
		self:UpdateDisplay()
		self:Say(string.format('Menu entry removed: |cff96b4ff%s|r', entry))

		return true
	end
end

function CustomMenuFu:Open(parent, ...)
	self:SetFontSizePercent(parent, 1.4)
end

function CustomMenuFu:OnTextUpdate()
	local title = self.db.profile.title or 'CustomMenuFu'
	self:SetText(title)
end

function CustomMenuFu:OnTooltipUpdate()
	self.title	= self.db.profile.title or 'CustomMenuFu'

	local c = tablet:AddCategory(
		-- 'text',			self.title,
		-- 'hideBlankLine',	true
		)

	c:AddLine()

	for name, script in pairs(self.db.profile.entries) do
		if name and script then
			if script == "---" then
				c:AddLine()
			else
				c:AddLine(
					"text", name,
					"func", function() RunScript(script) end
				)
			end
		end
	end
end

function CustomMenuFu:List()
	local entries = self.db.profile.entries

	if not entries or (entries == {}) then
		self:Say('you have not yet defined any entries for your CustomMenu')
		return false
	end

	self:Say('|cffffff00Listing currently defined entries:|r')

	local total = 0

	for name, text in pairs(entries) do
		self:Say(string.format('entry: |cff96b4ff%s|r # |cff28ff28%s|r', name, text))
		total = total + 1
	end

	self:Say(string.format('|cffffff00Total entries:|r |cff28ff28%s|r', total))

	return total
end

function CustomMenuFu:ShowEntry(entry)
	entry	= entry or ''

	if not self.db.profile.entries[entry] then
		self:Say(string.format('No such entry |cff96b4ff%s|r', entry))
		return false
	else
		self:Say(string.format('Entry |cff96b4ff%s|r: |cff28ff28%s|r', entry, self.db.profile.entries[entry]))
		return true
	end
end

--[[
function CustomMenuFu:GetHexColour(r, g, b)
	return true
end
]]


function CustomMenuFu:Say(msg)
	--[[
		Pretty print with a (potentially) coloured prefix
	]]

	local msg	= msg or ""
	msg		= strtrim(msg)

	if msg == "" then
		DEFAULT_CHAT_FRAME:AddMessage('CustomMenuFu:Say() called with a blank msg set')
		return false
	end

	return DEFAULT_CHAT_FRAME:AddMessage(string.format('|cffffffaaCustomMenuFu:|r |cffffffff%s|r', msg))
end
